#include <Arduino.h>
#include <functional>
#include "WifiControl.h"
#include "BrowserGui.h"
#include "LedControl.h"
#include "Timecontrol.h"
#include "AmbientLightControl.h"
#include "Storage.h"
#include <FastLED.h>


#define RETRY_TIME 10*60*1000 //10 minutes in milliseconds
#define CWC_VERSION 2.0

static TimeControl& timecontrol = TimeControl::getInstance();
static WifiControl& wificontrol = WifiControl::getInstance();
static BrowserGui& browsergui = BrowserGui::getInstance();
static LedControl& ledcontrol = LedControl::getInstance();
static AmbientLightControl& ambcontrol = AmbientLightControl::getInstance();

static long lastTry = 0;

/* indicator, if one time has passed, then update leds */
uint8_t last_minutes = 0;
uint8_t last_hours = 0;
uint8_t last_seconds = 0;
uint8_t lastPrintSeconds = 0;

void setup()
{
  Serial.begin(115200);
  Serial.println("");
  Serial.print("CWC Version: ");
  Serial.println(CWC_VERSION);

  storageInit();
  ambcontrol.start();
  ledcontrol.start();
  if (!wificontrol.connectWifi())
  {
    if (!wificontrol.connectWifi())
    {
      Serial.println("Save retry time");
      lastTry = millis();
    }
  }

  browsergui.start();
  timecontrol.start();
}

void loop()
{
  uint8_t hours = 0;
  uint8_t minutes = 0;
  uint8_t seconds = 0;

  browsergui.serverLoop();
  timecontrol.getTime(&hours, &minutes, &seconds);
  if (wificontrol.isWifiConnected())
  {
    if (last_minutes != minutes || last_hours != hours)
    {
      Serial.print("Time: ");
      Serial.print(hours);
      Serial.print(":");
      Serial.println(minutes);
      last_minutes = minutes;
      last_hours = hours;
      ledcontrol.setTime(hours, minutes);
    }
    if (last_seconds != seconds)
    {
      last_seconds = seconds;
      ambcontrol.updateBrightness();
    }
  }
  else
  {
    if (lastPrintSeconds != seconds)
    {
      lastPrintSeconds = seconds;
      Serial.print("No WIFI. Reconnect in ");
      Serial.println((RETRY_TIME - (millis() - lastTry)) / 1000);
    }
    if ((millis() - lastTry) > RETRY_TIME)
    {
      lastTry = millis();
      wificontrol.connectWifi();
    }
  }
}
